package BoutrosLab::TSVStream;

=head1 NAME

    BoutrosLab::TSVStream

=head1 SYNOPSIS

    See:
		BoutrosLab::TSVStream::IO
			for a collection of Moose Roles that can be
			consumed by a class to add a reader/writer method
			to build an object that can accept/generate a
			stream of objects of the consuming class as text
			from/to a file or stream.
		BoutrosLab::TSVStream::Format
			for pre-defined classes that have reader/writer
			methods.

=head1 VERSION

Version 0.5.0

=cut

our $VERSION = '0.5.0';

=head1 AUTHOR

John Macdonald - Boutros Lab

=head1 ACKNOWLEDGEMENTS

Paul Boutros, Phd, PI - Boutros Lab

The Ontario Institute for Cancer Research

=cut

1;

